@@ -1,5 +1,5 @@
 /*
- * $Id: helper.c 361 2005-11-16 23:35:45Z calrissian $
+ * $Id: helper.c 397 2006-01-28 21:11:50Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -46,19 +46,21 @@
 # include <errno.h>
 #endif
 #ifdef HAVE_CARES_H
-# include <arpa/nameser.h>
-# include <ares.h>
-int caport;
-unsigned long caadr;
-char *ca_tmpname;
-ares_channel channel;
-
-# ifndef RRFIXEDSZ
-#  define RRFIXEDSZ NS_RRFIXEDSZ
-#  define QFIXEDSZ  NS_QFIXEDSZ
-#  define HFIXEDSZ  NS_HFIXEDSZ
+# ifdef HAVE_ARPA_NAMESER_H
+#  include <arpa/nameser.h>
 # endif
-#endif
+# include <ares.h>
+# ifndef NS_RRFIXEDSZ
+#  define NS_RRFIXEDSZ 10
+#  define NS_QFIXEDSZ  4
+#  define NS_HFIXEDSZ  12
+# endif
+ int caport;
+ unsigned long caadr;
+ char *ca_tmpname;
+ ares_channel channel;
+
+#endif // HAVE_CARES_H
 
 #include "helper.h"
 #include "exit_code.h"
@@ -147,7 +149,7 @@ static const unsigned char *parse_rr(const unsigned char *aptr, const unsigned c
 		exit_code(2);
 	}
 	aptr += len;
-	if (aptr + RRFIXEDSZ > abuf + alen) {
+	if (aptr + NS_RRFIXEDSZ > abuf + alen) {
 		printf("error: not enough data in DNS answer 1\n");
 		free(name);
 		return NULL;
@@ -155,7 +157,7 @@ static const unsigned char *parse_rr(const unsigned char *aptr, const unsigned c
 	type = DNS_RR_TYPE(aptr);
 	dnsclass = DNS_RR_CLASS(aptr);
 	dlen = DNS_RR_LEN(aptr);
-	aptr += RRFIXEDSZ;
+	aptr += NS_RRFIXEDSZ;
 	if (aptr + dlen > abuf + alen) {
 		printf("error: not enough data in DNS answer 2\n");
 		free(name);
@@ -227,7 +229,7 @@ static const unsigned char *skip_rr(const unsigned char *aptr, const unsigned ch
 	status = ares_expand_name(aptr, abuf, alen, &name, &len);
 	aptr += len;
 	dlen = DNS_RR_LEN(aptr);
-	aptr += RRFIXEDSZ;
+	aptr += NS_RRFIXEDSZ;
 	aptr += dlen;
 	free(name);
 	return aptr;
@@ -243,7 +245,7 @@ static const unsigned char *skip_query(const unsigned char *aptr, const unsigned
 #endif
 	status = ares_expand_name(aptr, abuf, alen, &name, &len);
 	aptr += len;
-	aptr += QFIXEDSZ;
+	aptr += NS_QFIXEDSZ;
 	free(name);
 	return aptr;
 }
@@ -256,6 +258,12 @@ static void cares_callback(void *arg, int status, unsigned char *abuf, int alen)
 #ifdef DEBUG
 	printf("cares_callback: status=%i, alen=%i\n", status, alen);
 #endif
+	if (status != ARES_SUCCESS) {
+		if (verbose > 1)
+			printf("ares failed: %s\n", ares_strerror(status));
+		return;
+	}
+
 	ancount = DNS_HEADER_ANCOUNT(abuf);
 	nscount = DNS_HEADER_NSCOUNT(abuf);
 	arcount = DNS_HEADER_ARCOUNT(abuf);
@@ -264,14 +272,10 @@ static void cares_callback(void *arg, int status, unsigned char *abuf, int alen)
 	printf("ancount: %i, nscount: %i, arcount: %i\n", ancount, nscount, arcount);
 #endif
 
-	if (status != ARES_SUCCESS) {
-		printf("ares failed: %s\n", ares_strerror(status));
-		return;
-	}
 	/* safety check */
-	if (alen < HFIXEDSZ)
+	if (alen < NS_HFIXEDSZ)
 		return;
-	aptr = abuf + HFIXEDSZ;
+	aptr = abuf + NS_HFIXEDSZ;
 
 	aptr = skip_query(aptr, abuf, alen);
 
@@ -346,6 +350,7 @@ inline unsigned long srv_ares(char *host, int *port, char *srv) {
 	}
 	if (ca_tmpname != NULL)
 		free(ca_tmpname);
+	free(srvh);
 	return caadr;
 }
 #endif // HAVE_CARES_H
@@ -532,24 +537,26 @@ void get_fqdn(){
 #endif
 	}
 
-	he=gethostbyname(hname);
-	if (he) {
-		if (numeric == 1) {
-			snprintf(hlp, 15, "%s", inet_ntoa(*(struct in_addr *) he->h_addr_list[0]));
-			strncpy(fqdn, hlp, FQDN_SIZE);
-		}
-		else {
-			if ((strchr(he->h_name, '.'))!=NULL && (strchr(hname, '.'))==NULL) {
-				strncpy(fqdn, he->h_name, FQDN_SIZE);
+	if (!(numeric == 1 && is_ip(fqdn))) {
+		he=gethostbyname(hname);
+		if (he) {
+			if (numeric == 1) {
+				snprintf(hlp, 15, "%s", inet_ntoa(*(struct in_addr *) he->h_addr_list[0]));
+				strncpy(fqdn, hlp, FQDN_SIZE);
 			}
 			else {
-				strncpy(fqdn, hname, FQDN_SIZE);
+				if ((strchr(he->h_name, '.'))!=NULL && (strchr(hname, '.'))==NULL) {
+					strncpy(fqdn, he->h_name, FQDN_SIZE);
+				}
+				else {
+					strncpy(fqdn, hname, FQDN_SIZE);
+				}
 			}
 		}
-	}
-	else {
-		fprintf(stderr, "error: cannot resolve local hostname: %s\n", hname);
-		exit_code(2);
+		else {
+			fprintf(stderr, "error: cannot resolve local hostname: %s\n", hname);
+			exit_code(2);
+		}
 	}
 	if ((strchr(fqdn, '.'))==NULL) {
 		if (hostname) {
@@ -589,6 +596,62 @@ void replace_string(char *mess, char *search, char *replacement){
 	}
 }
 
+/* checks if the strings contains special double marks and then
+ * replace all occurences of this strings in the message */
+void replace_strings(char *mes, char *strings) {
+	char *pos, *atr, *val, *repl, *end;
+	char sep;
+
+	pos=atr=val=repl = NULL;
+#ifdef DEBUG
+	printf("replace_strings entered\nstrings: '%s'\n", strings);
+#endif
+	if ((isalnum(*strings) != 0) && 
+		(isalnum(*(strings + strlen(strings) - 1)) != 0)) {
+		replace_string(req, "$replace$", replace_str);
+	}
+	else {
+		sep = *strings;
+#ifdef DEBUG
+		printf("sep: '%c'\n", sep);
+#endif
+		end = strings + strlen(strings);
+		pos = strings + 1;
+		while (pos < end) {
+			atr = pos;
+			pos = strchr(atr, sep);
+			if (pos != NULL) {
+				*pos = '\0';
+				val = pos + 1;
+				pos = strchr(val, sep);
+				if (pos != NULL) {
+					*pos = '\0';
+					pos++;
+				}
+			}
+#ifdef DEBUG
+			printf("atr: '%s'\nval: '%s'\n", atr, val);
+#endif
+			if ((atr != NULL) && (val != NULL)) {
+				repl = str_alloc(strlen(val) + 3);
+				if (repl == NULL) {
+					printf("failed to allocate memory\n");
+					exit_code(2);
+				}
+				sprintf(repl, "$%s$", atr);
+				replace_string(mes, repl, val);
+				free(repl);
+			}
+#ifdef DEBUG
+			printf("pos: '%s'\n", pos);
+#endif
+		}
+	}
+#ifdef DEBUG
+	printf("mes:\n'%s'\n", mes);
+#endif
+}
+
 /* insert \r in front of all \n if it is not present allready */
 void insert_cr(char *mes){
 	char *lf, *pos, *backup;
@@ -715,14 +778,15 @@ int str_to_int(char *num)
 }
 
 /* reads into the given buffer from standard input until the EOF
- * character or the given size of the buffer is exceeded */
-int read_stdin(char *buf, int size)
+ * character, LF character or the given size of the buffer is exceeded */
+int read_stdin(char *buf, int size, int ret)
 {
 	int i, j;
 
 	for(i = 0; i < size - 1; i++) {
 		j = getchar();
-		if (j == EOF) {
+		if (((ret == 0) && (j == EOF)) ||
+			((ret == 1) && (j == '\n'))) {
 			*(buf + i) = '\0';
 			return i;
 		}
