@@ -1,5 +1,5 @@
 /*
- * $Id: shoot.c 369 2005-11-20 18:40:24Z calrissian $
+ * $Id: shoot.c 397 2006-01-28 21:11:50Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -262,14 +262,24 @@ void handle_default()
 		else if (verbose>0) {
 			printf("%s\n", rec);
 		}
-		else if (timing) {
-			printf("%.3f ms\n", deltaT(&(times.firstsendt), &(times.recvtime)));
+		if (timing > 0) {
+			timing--;
+			if (timing == 0) {
+				printf("%.3f/%.3f/%.3f ms\n", delays.small_delay, delays.all_delay / counters.run, delays.big_delay);
+			}
+			else {
+				counters.run++;
+				new_transaction(req);
+				delays.retryAfter = SIP_T1;
+			}
 		}
-		if (regexec(&(regexps.okexp), rec, 0, 0, 0) == REG_NOERROR) {
-			on_success(rec);
-		}
-		else {
-			exit_code(1);
+		if (timing == 0) {
+			if (regexec(&(regexps.okexp), rec, 0, 0, 0) == REG_NOERROR) {
+				on_success(rec);
+			}
+			else {
+				exit_code(1);
+			}
 		}
 	}
 }
@@ -775,7 +785,6 @@ void before_sending()
 /* this is the main function with the loops and modes */
 void shoot(char *buf, int buff_size)
 {
-	struct sockaddr_in	daddr;
 	struct timespec sleep_ms_s, sleep_rem;
 	int ret, cseqtmp, rand_tmp;
 	char buf2[BUFSIZE], buf3[BUFSIZE], lport_str[LPORT_STR_LEN];
@@ -793,7 +802,7 @@ void shoot(char *buf, int buff_size)
 
 	/* initalize local vars */
 	cdata.dontsend=cdata.dontrecv=counters.retrans_r_c=counters.retrans_s_c= 0;
-	delays.big_delay=counters.send_counter= 0;
+	delays.big_delay=counters.send_counter=counters.run= 0;
 	times.delaytime.tv_sec = 0;
 	times.delaytime.tv_usec = 0;
 	usern = NULL;
@@ -840,7 +849,7 @@ void shoot(char *buf, int buff_size)
 			replace_string(req, "$user$", username);
 	}
 	if (replace_str)
-		replace_string(req, "$replace$", replace_str);
+		replace_strings(req, replace_str);
 
 	/* set all regular expression to simplfy the result code indetification */
 	regcomp(&(regexps.replyexp), "^SIP/[0-9]\\.[0-9] [1-6][0-9][0-9]", 
@@ -936,7 +945,7 @@ void shoot(char *buf, int buff_size)
 			set_maxforw(req, maxforw);
 	}
 
-	cdata.connected = set_target(&daddr, address, rport, cdata.csock, cdata.connected);
+	cdata.connected = set_target(&(cdata.adr), address, rport, cdata.csock, cdata.connected);
 
 	/* here we go until someone decides to exit */
 	while(1) {
@@ -985,7 +994,18 @@ void shoot(char *buf, int buff_size)
 					}
 				else if (regexec(&(regexps.authexp), rec, 0, 0, 0) == REG_NOERROR) {
 					if (!username) {
-						fprintf(stderr, "%s\nerror: received 401 but cannot "
+						if (timing > 0) {
+							timing--;
+							if (timing == 0) {
+								printf("%.3f/%.3f/%.3f ms\n", delays.small_delay, delays.all_delay / counters.run, delays.big_delay);
+								exit_code(0);
+							}
+							counters.run++;
+							new_transaction(req);
+							delays.retryAfter = SIP_T1;
+							continue;
+						}
+						fprintf(stderr, "%s\nerror: received 40[17] but cannot "
 							"authentication without a username\n", rec);
 						exit_code(2);
 					}
@@ -999,7 +1019,7 @@ void shoot(char *buf, int buff_size)
 				} /* if auth...*/
 				/* lets see if received a redirect */
 				if (redirects == 1 && regexec(&(regexps.redexp), rec, 0, 0, 0) == REG_NOERROR) {
-					handle_3xx(&daddr);
+					handle_3xx(&(cdata.adr));
 				} /* if redircts... */
 				else if (trace == 1) {
 					trace_reply();
