@@ -1,5 +1,5 @@
 /*
- * $Id: sipsak.c 361 2005-11-16 23:35:45Z calrissian $
+ * $Id: sipsak.c 397 2006-01-28 21:11:50Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -142,12 +142,13 @@ void print_long_help() {
 		"  --disposition=STRING       Content-Disposition value\n"
 		"  --search=REGEXP            search for a RegExp in replies and return error\n"
 		"                             on failfure\n"
-		"  --timing                   print the timing informations at the end\n"
+		"  --timing=NUMBER            number of test runs and print just the timings\n"
 		"  --symmetric                send and received on the same port\n"
 		"  --from=SIPURI              use the given uri as From in MESSAGE\n"
 		"  --timeout-factor=NUMBER    timeout multiplier for INVITE transactions\n"
 		"                             and reliable transports (default: 64)\n"
 		"  --transport=STRING         specify transport to be used\n"
+		"  --headers=STRING           adds additional headers to the request\n"
 		);
 	exit_code(0);
 }
@@ -207,12 +208,13 @@ void print_help() {
 		"  -B STRING         send a message with string as body\n"
 		"  -O STRING         Content-Disposition value\n"
 		"  -P NUMBER         Number of processes to start\n"
-		"  -A                print timing informations\n"
+		"  -A NUMBER         number of test runs and print just timings\n"
 		"  -S                use same port for receiving and sending\n"
 		"  -c SIPURI         use the given uri as From in MESSAGE\n"
 		"  -D NUMBER         timeout multiplier for INVITE transactions\n"
 		"                    and reliable transports (default: 64)\n"
 		"  -E STRING         specify transport to be used\n"
+		"  -j STRING         adds additional headers to the request\n"
 		);
 		exit_code(0);
 }
@@ -269,11 +271,12 @@ int main(int argc, char *argv[])
 		{"processes", 1, 0, 'P'},
 		{"auth-username", 1, 0, 'u'},
 		{"no-crlf", 0, 0, 'L'},
-		{"timing", 0, 0, 'A'},
+		{"timing", 1, 0, 'A'},
 		{"symmetric", 0, 0, 'S'},
 		{"from", 1, 0, 'c'},
 		{"timeout-factor", 1, 0, 'D'},
 		{"transport", 1, 0, 'E'},
+		{"headers", 1, 0, 'j'},
 		{0, 0, 0, 0}
 	};
 #endif
@@ -284,7 +287,7 @@ int main(int argc, char *argv[])
 	namebeg=nameend=maxforw= -1;
 	numeric=via_ins=redirects=fix_crlf=processes = 1;
 	username=password=replace_str=hostname=contact_uri=mes_body = NULL;
-	con_dis=auth_username=from_uri = NULL;
+	con_dis=auth_username=from_uri=headers = NULL;
 	scheme = user = host = backup = req = rep = rec = NULL;
 	re = NULL;
 	address= 0;
@@ -301,17 +304,26 @@ int main(int argc, char *argv[])
 
 	/* lots of command line switches to handle*/
 #ifdef HAVE_GETOPT_LONG
-	while ((c=getopt_long(argc, argv, "a:Ab:B:c:C:dD:e:E:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:Xz:", l_opts, &option_index)) != EOF){
+	while ((c=getopt_long(argc, argv, "a:A:b:B:c:C:dD:e:E:f:Fg:GhH:iIj:l:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:Xz:", l_opts, &option_index)) != EOF){
 #else
-	while ((c=getopt(argc, argv, "a:Ab:B:c:C:dD:e:E:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:z:")) != EOF){
+	while ((c=getopt(argc, argv, "a:A:b:B:c:C:dD:e:E:f:Fg:GhH:iIj:l:Lm:MnNo:O:p:P:q:r:Rs:St:Tu:UvVwW:x:z:")) != EOF){
 #endif
 		switch(c){
 			case 'a':
-				password=str_alloc(strlen(optarg) + 1);
-				strncpy(password, optarg, strlen(optarg));
+				if (strlen(optarg) == 1 && STRNCASECMP(optarg, "-", 1) == 0) {
+					password = str_alloc(SIPSAK_MAX_PASSWD_LEN);
+					printf("Please enter the password (max. length %i): ", SIPSAK_MAX_PASSWD_LEN);
+					if (read_stdin(password, SIPSAK_MAX_PASSWD_LEN, 1) == 0) {
+						exit_code(0);
+					}
+				}
+				else {
+					password=str_alloc(strlen(optarg) + 1);
+					strncpy(password, optarg, strlen(optarg));
+				}
 				break;
 			case 'A':
-				timing=1;
+				timing=str_to_int(optarg);
 				break;
 			case 'b':
 				namebeg=str_to_int(optarg);
@@ -426,7 +438,7 @@ int main(int argc, char *argv[])
 					fclose(pf);
 				}
 				else if (strlen(optarg) == 1 && STRNCASECMP(optarg, "-", 1) == 0) {
-					if (read_stdin(&buff[0], sizeof(buff)) == 0) {
+					if (read_stdin(&buff[0], sizeof(buff), 0) == 0) {
 						exit_code(0);
 					}
 				}
@@ -454,6 +466,9 @@ int main(int argc, char *argv[])
 			case 'I':
 				invite=1;
 				break;
+			case 'j':
+				headers=optarg;
+				break;
 			case 'l':
 				lport=str_to_int(optarg);
 				break;
@@ -642,6 +657,9 @@ int main(int argc, char *argv[])
 				printf(", INTERNAL_MD5");
 # endif
 #endif
+#ifdef HAVE_OPENSSL_SHA1
+				printf(", OPENSSL_SHA1");
+#endif
 #ifdef HAVE_CARES_H
 				printf(", SRV_SUPPORT(ARES)");
 #else
@@ -701,6 +719,18 @@ int main(int argc, char *argv[])
 	if ((file_b) && (fix_crlf)) {
 		insert_cr(buff);
 	}
+	if (headers) {
+		backup = str_alloc(strlen(headers) + 30); // FIXME
+		strcpy(backup, headers);
+		headers = backup;
+		replace_string(headers, "\\n", "\r\n");
+		backup = headers + strlen(headers) - 1;
+		if (*backup != '\n') {
+			strcpy(backup + 1, "\r\n");
+		}
+		if (file_b)
+			insert_header(buff, headers, 1);
+	}
 	/* lots of conditions to check */
 	if (trace) {
 		if (usrloc || flood || randtrash) {
