@@ -1,5 +1,5 @@
 /*
- * $Id: sipsak.h 361 2005-11-16 23:35:45Z calrissian $
+ * $Id: sipsak.h 397 2006-01-28 21:11:50Z calrissian $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004-2005 Nils Ohlmeier
@@ -82,6 +82,12 @@
 # endif
 #endif
 
+#ifdef HAVE_OPENSSL_SHA_H
+# ifdef HAVE_CRYPTO_WITH_SHA1
+#  define HAVE_OPENSSL_SHA1
+# endif
+#endif
+
 #ifndef REG_NOERROR
 # define REG_NOERROR 0
 #endif
@@ -107,6 +113,8 @@
 #define SIPSAK_VERSION PACKAGE_VERSION
 #define BUFSIZE		4096
 
+#define SIPSAK_MAX_PASSWD_LEN 20
+
 #define REQ_REG 1
 #define REQ_REM 2
 #define REQ_INV 3
@@ -227,8 +235,14 @@
 #define PROXYAUZ_STR "Proxy-Authorization: Digest "
 #define PROXYAUZ_STR_LEN (sizeof(PROXYAUZ_STR) - 1)
 
-#define ALGO_MD5_STR "algorithm=MD5, "
-#define ALGO_MD5_STR_LEN (sizeof(ALGO_MD5_STR) - 1)
+#define ALGO_STR "algorithm="
+#define ALGO_STR_LEN (sizeof(ALGO_STR) - 1)
+
+#define MD5_STR "MD5, "
+#define MD5_STR_LEN (sizeof(MD5_STR) - 1)
+
+#define SHA1_STR "SHA1, "
+#define SHA1_STR_LEN (sizeof(SHA1_STR) - 1)
 
 #define REALM_STR "realm="
 #define REALM_STR_LEN (sizeof(REALM_STR) - 1)
@@ -269,8 +283,16 @@
 #define USRLOC_EXP_DEF 15
 #define FLOOD_METH "OPTIONS"
 
-#define MD5_HASHLEN 16
-#define HASHHEXLEN 2 * MD5_HASHLEN
+#define SIPSAK_HASHLEN_MD5 16
+#define SIPSAK_HASHHEXLEN_MD5 2 * SIPSAK_HASHLEN_MD5 
+#ifdef HAVE_OPENSSL_SHA1
+# define SIPSAK_HASHLEN_SHA1 20
+# define SIPSAK_HASHLEN SIPSAK_HASHLEN_SHA1 
+# define SIPSAK_HASHHEXLEN_SHA1 2 * SIPSAK_HASHLEN_SHA1 
+#else
+# define SIPSAK_HASHLEN SIPSAK_HASHLEN_MD5 
+#endif
+#define SIPSAK_HASHHEXLEN 2 * SIPSAK_HASHLEN
 
 /* lots of global variables. ugly but makes life easier. */
 unsigned long address;
@@ -281,7 +303,7 @@ int maxforw, lport, rport, randtrash, trashchar, numeric, symmetric;
 int file_b, uri_b, trace, via_ins, usrloc, redirects, rand_rem, replace_b;
 int empty_contact, nagios_warn, fix_crlf, timing, outbound_proxy, inv_final;
 char *username, *domainname, *password, *replace_str, *hostname, *contact_uri;
-char *mes_body, *con_dis, *auth_username, *from_uri;
+char *mes_body, *con_dis, *auth_username, *from_uri, *headers;
 char fqdn[FQDN_SIZE];
 char target_dot[INET_ADDRSTRLEN], source_dot[INET_ADDRSTRLEN];
 char *req, *rep, *rec, *transport_str;
